unit main;

interface

uses
  Windows, DateUtils, apiPlugin, AIMPCustomPlugin, apiMenu, apiCore,
  apiActions, apiPlaylists, apiMessages;

type
  TAIMPMenuService = class(TObject)
  private
    function GetBuiltInMenu(ID: Integer): IAIMPMenuItem;
  public
    constructor Create;
  end;

  TAIMPMenuItemEventHandler = class(TInterfacedObject, IAIMPActionEvent)
  public
    procedure OnExecute(Data: IInterface); stdcall;
  end;

  TAIMPDemo = class(TAIMPCustomPlugin, IAIMPMessageHook)
  private
    //
  protected
    { TAIMPCustomPlugin }
    function InfoGet(Index: Integer): PWideChar; override; stdcall;
    function InfoGetCategories: Cardinal; override; stdcall;
    function Initialize(Core: IAIMPCore): HRESULT; override; stdcall;
    procedure Finalize; override; stdcall;
    { IAIMPMessageHook }
    procedure CoreMessage(Message: DWORD; Param1: Integer; Param2: Pointer;
      var Result: HRESULT); stdcall;
  end;

var
  FEnable: Boolean;
  FCurrentGroup: Integer;
  FCurrentPlaylist: IAIMPPlaylist;

implementation

uses
  ActiveX, apiObjects, apiWrappers, apiPlayer, sysutils;

function GetItemGroup(const PlaylistItem: IAIMPPlaylistItem): Integer;
var
  APlaylistGroup: IAIMPPlaylistGroup;
begin
  if Succeeded(PlaylistItem.GetValueAsObject(AIMP_PLAYLISTITEM_PROPID_GROUP,
    IID_IAIMPPlaylistGroup, APlaylistGroup)) then
    APlaylistGroup.GetValueAsInt32(AIMP_PLAYLISTGROUP_PROPID_INDEX, Result);
end;

function GetLastGroupItem(const PlaylistItem: IAIMPPlaylistItem)
  : IAIMPPlaylistItem;
var
  APlaylistGroup: IAIMPPlaylistGroup;
  ACount, ASwitch: Integer;
begin
  if Succeeded(PlaylistItem.GetValueAsObject(AIMP_PLAYLISTITEM_PROPID_GROUP,
    IID_IAIMPPlaylistGroup, APlaylistGroup)) then
  begin
    ACount := APlaylistGroup.GetItemCount;

    while ACount > 0 do
    begin
      ASwitch := 0;
      ACount := ACount - 1;
      if Succeeded(APlaylistGroup.GetItem(ACount, IID_IAIMPPlaylistItem, Result))
      then
        Result.GetValueAsInt32(AIMP_PLAYLISTITEM_PROPID_PLAYINGSWITCH, ASwitch);

      if ASwitch <> 0 then Break;
    end;
  end;
end;

function TAIMPMenuService.GetBuiltInMenu(ID: Integer): IAIMPMenuItem;
var
  AMenuService: IAIMPServiceMenuManager;
begin
  if CoreGetService(IAIMPServiceMenuManager, AMenuService) then
    try
      CheckResult(AMenuService.GetBuiltIn(ID, Result));
    finally
      AMenuService := nil;
    end;
end;

constructor TAIMPMenuService.Create;
var
  AMenuItem: IAIMPMenuItem;
begin
  if Succeeded(CoreIntf.CreateObject(IID_IAIMPMenuItem, AMenuItem)) then
  begin
    with AMenuItem do
    begin
      CheckResult(SetValueAsObject(AIMP_MENUITEM_PROPID_ID,
        MakeString('awa.stopaftergroup.enable')));
      CheckResult(SetValueAsObject(AIMP_MENUITEM_PROPID_NAME,
        MakeString('Stop After Current Group')));
      CheckResult(SetValueAsInt32(AIMP_MENUITEM_PROPID_CHECKED, 0));
      CheckResult(SetValueAsObject(AIMP_MENUITEM_PROPID_EVENT,
        TAIMPMenuItemEventHandler.Create));
      CheckResult(SetValueAsObject(AIMP_MENUITEM_PROPID_PARENT,
        GetBuiltInMenu(AIMP_MENUID_PLAYER_PLAYLIST_MISCELLANEOUS)));
      CheckResult(SetValueAsInt32(AIMP_MENUITEM_PROPID_STYLE,
        AIMP_MENUITEM_STYLE_CHECKBOX));
    end;
    CheckResult(CoreIntf.RegisterExtension(IID_IAIMPServiceMenuManager,
      AMenuItem));
  end;
end;

procedure TAIMPMenuItemEventHandler.OnExecute(Data: IInterface);
  procedure Disable(const MenuItem: IAIMPMenuItem);
  begin
    FEnable := False;
    FCurrentGroup := -1;
    FCurrentPlaylist := nil;

    MenuItem.SetValueAsInt32(AIMP_MENUITEM_PROPID_CHECKED, 0);
  end;

var
  AMenuItem: IAIMPMenuItem;
  APropertyList: IAIMPPropertyList;
  APlayer: IAIMPServicePlayer;
  APlaylist: IAIMPPlaylist;
  APlaylistItem: IAIMPPlaylistItem;
  AGroupping, AChecked: Integer;
  AStop: LongBool;
begin
  AMenuItem := Data as IAIMPMenuItem;

  if Succeeded(AMenuItem.GetValueAsInt32(AIMP_MENUITEM_PROPID_CHECKED, AChecked))
  then
    if AChecked <> 0 then
    begin
      if CoreGetService(IAIMPServicePlayer, APlayer) then
        if Succeeded(APlayer.GetPlaylistItem(APlaylistItem)) then
        begin
          APlaylistItem.GetValueAsObject(AIMP_PLAYLISTITEM_PROPID_PLAYLIST,
            IID_IAIMPPlaylist, APlaylist);
          APlaylist.QueryInterface(IID_IAIMPPropertyList, APropertyList);
          //       
          APropertyList.GetValueAsInt32(AIMP_PLAYLIST_PROPID_GROUPPING,
            AGroupping);
          if AGroupping <> 0 then
          begin
            FCurrentGroup := GetItemGroup(APlaylistItem);
            FCurrentPlaylist := APlaylist;
            FEnable := True;

            if GetLastGroupItem(APlaylistItem) = APlaylistItem then
            begin
              AStop := True;
              MessageDispatcherSetPropValue
                (AIMP_MSG_PROPERTY_STOP_AFTER_TRACK, @AStop);
              Disable(AMenuItem);
            end;
          end
          else
            //    
            Disable(AMenuItem);
        end
        else
          //      
          Disable(AMenuItem);
    end
    else
    begin
      AStop := False;
      MessageDispatcherSetPropValue(AIMP_MSG_PROPERTY_STOP_AFTER_TRACK, @AStop);
      Disable(AMenuItem);
    end;
end;

procedure TAIMPDemo.CoreMessage(Message: DWORD; Param1: Integer;
  Param2: Pointer; var Result: HRESULT);
  procedure Disable;
  var
    AMenuService: IAIMPServiceMenuManager;
    AMenuItem: IAIMPMenuItem;
  begin
    FEnable := False;
    FCurrentGroup := -1;
    FCurrentPlaylist := nil;

    if CoreGetService(IAIMPServiceMenuManager, AMenuService) then
      if Succeeded(AMenuService.GetByID(MakeString('awa.stopaftergroup.enable'),
        AMenuItem)) then
        CheckResult(AMenuItem.SetValueAsInt32(AIMP_MENUITEM_PROPID_CHECKED, 0));
  end;

var
  APlayer: IAIMPServicePlayer;
  APlaylistItem: IAIMPPlaylistItem;
  APlaylist: IAIMPPlaylist;
  AStop: LongBool;
begin
  case Message of
    AIMP_MSG_EVENT_STREAM_START:
      begin
        if not FEnable then
          Exit;

        if CoreGetService(IAIMPServicePlayer, APlayer) then
          if Succeeded(APlayer.GetPlaylistItem(APlaylistItem)) then
          begin
            APlaylistItem.GetValueAsObject(AIMP_PLAYLISTITEM_PROPID_PLAYLIST,
              IID_IAIMPPlaylist, APlaylist);
            //     - 
            if (FCurrentPlaylist <> APlaylist) or
              (FCurrentGroup <> GetItemGroup(APlaylistItem)) then
            begin
              Disable;
              Exit;
            end;
            //        - 
            if GetLastGroupItem(APlaylistItem) = APlaylistItem then
            begin
              AStop := True;
              MessageDispatcherSetPropValue
                (AIMP_MSG_PROPERTY_STOP_AFTER_TRACK, @AStop);
              Disable;
            end;
          end;
      end;
  end;
end;

function TAIMPDemo.Initialize(Core: IAIMPCore): HRESULT;
var
  ADispatcher: IAIMPServiceMessageDispatcher;
  APlaylistManager: IAIMPServicePlaylistManager;
  MenuService: TAIMPMenuService;
begin
  Result := Core.QueryInterface(IID_IAIMPServicePlaylistManager,
    APlaylistManager);
  if Succeeded(Result) then
  begin
    Result := inherited Initialize(Core);
    if Succeeded(Result) then
    begin
      try
        if Succeeded(Core.QueryInterface(IAIMPServiceMessageDispatcher,
          ADispatcher)) then
          CheckResult(ADispatcher.Hook(Self));
        MenuService := TAIMPMenuService.Create;
        FCurrentGroup := -1;
        FEnable := False;
      finally
        ADispatcher := nil;
        APlaylistManager := nil;
        FreeAndNil(MenuService);
      end;
    end;
  end;
end;

function TAIMPDemo.InfoGet(Index: Integer): PWideChar;
begin
  case Index of
    AIMP_PLUGIN_INFO_NAME:
      Result := 'StopAfterCurrentGroup 0.4';
    AIMP_PLUGIN_INFO_AUTHOR:
      Result := 'Awakunar';
    AIMP_PLUGIN_INFO_SHORT_DESCRIPTION:
      Result := '';
  else
    Result := nil;
  end;
end;

function TAIMPDemo.InfoGetCategories: Cardinal;
begin
  Result := AIMP_PLUGIN_CATEGORY_ADDONS;
end;

procedure TAIMPDemo.Finalize;
var
  ADispatcher: IAIMPServiceMessageDispatcher;
begin
  if Succeeded(CoreIntf.QueryInterface(IAIMPServiceMessageDispatcher,
    ADispatcher)) then
    ADispatcher.Unhook(Self);
  ADispatcher := nil;
end;

end.
